# -*- coding: utf-8 -*-

"""Add providers for Crop Ontology entries."""

import requests

import bioregistry


def main():
    """Run the script."""
    r = dict(bioregistry.read_registry())
    for prefix, resource in r.items():
        if not prefix.startswith("co_"):
            continue
        if not resource.example:
            print(prefix, "missing example")
            continue
        if resource.url:
            print(prefix, "has url", resource.url)
            url = bioregistry.get_link(prefix, resource.example)
            res = requests.get(url)
            print(res.text)
            print()
            continue
        resource.url = f"https://www.cropontology.org/rdf/{prefix.upper()}:$1"
    bioregistry.write_registry(r)


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-

"""Add examples from OLS."""

import random
from typing import Mapping

import requests
from tqdm import tqdm

import bioregistry


def _get_example(prefix: str, size: int = 1000):
    url = f"https://www.ebi.ac.uk/ols/api/ontologies/{prefix}/terms?page=1&size={size}"
    return _get_example_helper(prefix, url)


def _get_example_helper(prefix: str, url: str) -> str:
    if "page=50" in url:
        tqdm.write(f"[{prefix} too deep")
        raise KeyError
    tqdm.write(f"[{prefix}] getting {url}")
    res = requests.get(url).json()
    embedded = res.get("_embedded")
    if not embedded:
        tqdm.write(f"[{prefix} failure")
        tqdm.write(str(res))
        raise KeyError
    for term in embedded["terms"]:
        if term["short_form"].startswith(prefix.upper()):
            return term["short_form"]
    next_url = res["_links"]["next"]["href"]
    return _get_example_helper(prefix, next_url)


SKIP = {"afo", "dicom", "ensemblglossary", "co_321:root", "co_336", "co_359", "gexo", "hcao"}


def _get_missing_ols() -> Mapping[str, str]:
    """Get a map of prefixes to OLS prefixes that need checking."""
    return {
        prefix: bioregistry.get_ols_prefix(prefix)  # type: ignore
        for prefix, resource in bioregistry.read_registry().items()
        if (
            prefix not in SKIP
            and not bioregistry.has_no_terms(prefix)
            and resource.ols
            and not bioregistry.get_example(prefix)
        )
    }


def main():
    """Get OLS examples."""
    r = dict(bioregistry.read_registry())
    x = sorted(list(_get_missing_ols().items()))
    random.shuffle(x)
    for prefix, ols_prefix in tqdm(x[:30]):

        try:
            res = _get_example(ols_prefix, size=1000)
        except KeyError:
            continue
        if res is None:
            tqdm.write(f"no own terms in {prefix} (ols={ols_prefix})")
        else:
            tqdm.write(f"[{prefix}] got example {res}")
            if res.startswith(prefix.upper() + "_"):
                res = res[len(prefix) + 1 :]
            if res.startswith(prefix.upper() + ":"):
                res = res[len(prefix) + 1 :]
            r[prefix]["example"] = res
    bioregistry.write_registry(r)


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-

"""Add descriptions from a google curation sheet."""

import pandas as pd

import bioregistry

URL = (
    "https://docs.google.com/spreadsheets/d/e/2PACX-1vQVw4odnZF34f267p9WqdQOhi"
    "Y9tewD-jbnATgpi5W9smbkemvbOcVZSdeboXknoWxDhPyvtcxUYiQO/pub?gid=1947246172&single=true&output=tsv"
)


def main():
    """Add descriptions from a google curation sheet."""
    df = pd.read_csv(URL, sep="\t")
    del df[df.columns[0]]
    df = df[df.description.notna()]
    df = df[df["prefix"].map(lambda p: bioregistry.get_description(p) is None)]
    df = df[df["prefix"].map(lambda p: bioregistry.get_obofoundry_prefix(p) is None)]
    print(df.to_markdown())
    r = dict(bioregistry.read_registry())
    for prefix, description in df[["prefix", "description"]].values:
        r[prefix].description = description
    bioregistry.write_registry(r)


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-

"""Functionality for parsing IRIs."""

import collections.abc
from typing import Mapping, Sequence, Tuple, Union

from .resolve import get_format_urls, parse_curie

__all__ = [
    "parse_iri",
]


def _sort_key(kv):
    return -len(kv[0])


#: A prefix map in reverse sorted order based on length of the URL
#: in order to avoid conflicts of sub-URIs (thanks to Nico Matentzoglu for the idea)
PREFIX_MAP = sorted(get_format_urls().items(), key=_sort_key)

OLS_URL_PREFIX = "https://www.ebi.ac.uk/ols/ontologies/"
BIOREGISTRY_PREFIX = "https://bioregistry.io"
OBO_PREFIX = "http://purl.obolibrary.org/obo/"
IDOT_HTTPS_PREFIX = "https://identifiers.org/"
IDOT_HTTP_PREFIX = "http://identifiers.org/"
N2T_PREFIX = "https://n2t.net/"


def parse_iri(
    iri: str, prefix_map: Union[None, Mapping[str, str], Sequence[Tuple[str, str]]] = None
) -> Union[Tuple[str, str], Tuple[None, None]]:
    """Parse a compact identifier from an IRI.

    :param iri: A valid IRI
    :param prefix_map:
        If None, will use the default prefix map. If a mapping, will convert into a sorted
        list using ``sorted(prefix_map.items(), key=lambda kv: -len(kv[0]))``. If you plan
        to use this function in a loop, pre-compute this and pass it instead.
        If a list of pairs is passed, will use it directly.
    :return: A pair of prefix/identifier, if can be parsed

    IRI from an OBO PURL:

    >>> parse_iri("http://purl.obolibrary.org/obo/DRON_00023232")
    ('dron', '00023232')

    IRI from the OLS:

    >>> parse_iri("https://www.ebi.ac.uk/ols/ontologies/ecao/terms?iri=http://purl.obolibrary.org/obo/ECAO_0107180")
    ('ecao', '0107180')

    .. todo:: IRI from bioportal

    IRI from native provider

    >>> parse_iri("https://www.alzforum.org/mutations/1234")
    ('alzforum.mutation', '1234')

    Dog food:

    >>> parse_iri("https://bioregistry.io/DRON:00023232")
    ('dron', '00023232')

    IRIs from Identifiers.org (https and http, colon and slash):

    >>> parse_iri("https://identifiers.org/aop.relationships:5")
    ('aop.relationships', '5')
    >>> parse_iri("http://identifiers.org/aop.relationships:5")
    ('aop.relationships', '5')
    >>> parse_iri("https://identifiers.org/aop.relationships/5")
    ('aop.relationships', '5')
    >>> parse_iri("http://identifiers.org/aop.relationships/5")
    ('aop.relationships', '5')

    IRI from N2T
    >>> parse_iri("https://n2t.net/aop.relationships:5")
    ('aop.relationships', '5')

    .. todo:: IRI with weird embedding, like ones that end in .html
    """
    if isinstance(prefix_map, collections.abc.Mapping):
        prefix_map = sorted(prefix_map.items(), key=_sort_key)
    if iri.startswith(BIOREGISTRY_PREFIX):
        curie = iri[len(BIOREGISTRY_PREFIX) :]
        return parse_curie(curie)
    if iri.startswith(OLS_URL_PREFIX):
        sub_iri = iri.rsplit("=", 1)[1]
        return parse_obolibrary_purl(sub_iri)
    if iri.startswith(OBO_PREFIX):
        return parse_obolibrary_purl(iri)
    if iri.startswith(IDOT_HTTPS_PREFIX):
        curie = iri[len(IDOT_HTTPS_PREFIX) :]
        return _safe_parse_curie(curie)
    if iri.startswith(IDOT_HTTP_PREFIX):
        curie = iri[len(IDOT_HTTP_PREFIX) :]
        return _safe_parse_curie(curie)
    if iri.startswith(N2T_PREFIX):
        curie = iri[len(N2T_PREFIX) :]
        return parse_curie(curie)
    for prefix, prefix_url in prefix_map or PREFIX_MAP:
        if iri.startswith(prefix_url):
            return prefix, iri[len(prefix_url) :]
    return None, None


def _safe_parse_curie(curie: str) -> Union[Tuple[str, str], Tuple[None, None]]:
    for sep in "_/:":
        prefix, identifier = parse_curie(curie, sep)
        if prefix is not None and identifier is not None:
            return prefix, identifier
    return None, None


def parse_obolibrary_purl(iri: str) -> Union[Tuple[str, str], Tuple[None, None]]:
    """Parse an OBO Library PURL.

    :param iri: A valid IRI
    :return: A pair of prefix/identifier, if can be parsed

    >>> parse_obolibrary_purl("http://purl.obolibrary.org/obo/DRON_00023232")
    ('dron', '00023232')

    >>> parse_obolibrary_purl("http://purl.obolibrary.org/obo/FBbt_0000001")
    ('fbbt', '0000001')
    """
    curie = iri[len("http://purl.obolibrary.org/obo/") :]
    return parse_curie(curie, sep="_")


def _main():
    """Run this as ``python -m bioregistry.parse_iri`` to get a list of IRIs that can be constructed, but not parsed."""
    import bioregistry
    from tabulate import tabulate

    rows = []
    for prefix in bioregistry.read_registry():
        example = bioregistry.get_example(prefix)
        if example is None:
            continue
        iri = bioregistry.get_link(prefix, example, use_bioregistry_io=False)
        if iri is None:
            # print('no iri for', prefix, example)
            continue
        k, v = parse_iri(iri)
        if k is None:
            rows.append((prefix, example, iri, k, v))
    print(tabulate(rows))


if __name__ == "__main__":
    _main()
